@@ -53,7 +53,6 @@
 #include "glib-utils.h"
 #include "gconf-utils.h"
 #include "file-utils.h"
-#include "jpeg-utils.h"
 #include "pixbuf-utils.h"
 #include "typedefs.h"
 
@@ -624,7 +623,17 @@ get_mime_type_from_ext (const char *ext)
 
 gboolean mime_type_is_image (const char *mime_type)
 {
-	return (   (strstr (mime_type, "image") != NULL)
+	/* Valid image mime types:
+	   	1. All image* types, 
+		1b.	except for image/x-xcf, image/x-compressed-xcf
+			because we can't read these gimp files, and fast,
+			reliable converters are not really available.
+		2. application/x-crw
+			This is a RAW photo file, which for some reason
+			uses an "application" prefix instead of "image".
+	*/
+
+	return (   ((strstr (mime_type, "image") != NULL) && (strstr (mime_type, "xcf") == NULL))
 		|| (strcmp (mime_type, "application/x-crw") == 0) );
 }
 
@@ -644,7 +653,8 @@ gboolean file_is_image (const gchar *name,
 
 gboolean mime_type_is_video (const char *mime_type)
 {
-	return (strstr (mime_type, "video") != NULL);
+	return ( (strstr (mime_type, "video") != NULL) ||
+		 (strcmp (mime_type, "application/ogg") == 0));
 }
 
 
@@ -850,17 +860,20 @@ get_file_mime_type (const char *filename,
 	if (extension != NULL) {
 
 		/* Raw NEF files are sometimes mis-recognized as tiff files. Fix that. */
-		if (!strcmp (result, "image/tiff") && !strcasecmp (extension, "nef"))
+		if (!strcmp_null_tollerant (result, "image/tiff") && 
+		    !strcasecmp (extension, "nef"))
 			return "image/x-nikon-nef";
 
 		/* Raw CR2 files are sometimes mis-recognized as tiff files. Fix that. */
-		if (!strcmp (result, "image/tiff") && !strcasecmp (extension, "cr2"))
-			return "image/x-canon-crw";
+		if (!strcmp_null_tollerant (result, "image/tiff") && 
+		    !strcasecmp (extension, "cr2"))
+			return "image/x-canon-cr2";
 
 		/* Check unrecognized binary types for special types that are not
 		   handled correctly in the normal mime databases. */
 
-		if ((result == NULL) || (strcmp (result, "application/octet-stream") == 0)) {
+		if ((result == NULL) || 
+		    (strcmp_null_tollerant (result, "application/octet-stream") == 0)) {
 
 			/* If the file extension is not recognized, or the content is
 			   determined to be binary data (octet-stream), check for HDR file
@@ -924,6 +937,7 @@ mime_type_is_raw (const char *mime_type)
 		|| mime_type_is (mime_type, "image/x-dcraw")		/* dcraw */
 		|| mime_type_is (mime_type, "image/x-minolta-mrw")  	/* freedesktop.org.xml */
 		|| mime_type_is (mime_type, "image/x-canon-crw")    	/* freedesktop.org.xml */
+		|| mime_type_is (mime_type, "image/x-canon-cr2")    	/* freedesktop.org.xml */
 		|| mime_type_is (mime_type, "image/x-nikon-nef")	/* freedesktop.org.xml */
 		|| mime_type_is (mime_type, "image/x-kodak-dcr")    	/* freedesktop.org.xml */
 		|| mime_type_is (mime_type, "image/x-kodak-kdc")    	/* freedesktop.org.xml */
@@ -2091,7 +2105,7 @@ escape_uri (const char *uri)
 /* characters to escape */
 static gchar bad_char[] = { '$', '\'', '`', '"', '\\', '!', '?', '*',
 			    ' ', '(', ')', '[', ']', '&', '|', '@' , '#',
-			    ';' };
+			    ';', '<', '>' };
 
 /* the size of bad_char */
 static const gint bad_chars = sizeof (bad_char) / sizeof (gchar);
@@ -2463,45 +2477,94 @@ prune_cache (void)
 }
 
 
+long
+get_space_used_in_kb (const char *path)
+{
+	char   *command;
+	char   *command_output = NULL;
+	long 	space_used = 0;
+
+        command = g_strdup_printf ("du -ks %s", path);
+        g_spawn_command_line_sync (command,
+                                   &command_output,
+                                   NULL, NULL, NULL);
+	g_free (command);
+
+        if (command_output != NULL) {
+                space_used = strtol (command_output, NULL, 10);
+                g_free (command_output);
+        }
+
+        return space_used;
+}
+
+#define MAX_CACHE_SIZE_IN_KB	(256 * 1024)
+
+void
+check_cache_space (void)
+{
+	char   *command;
+	char   *cache_dir;
+	int	i = 0;
+
+	cache_dir = g_strdup_printf ("%s/%s/",g_get_home_dir (),RC_REMOTE_CACHE_DIR);
+	if (cache_dir == NULL)
+		return;
+
+	command = g_strdup_printf ("cd %s; ls -t1 %s | tail -n 1 | xargs rm", cache_dir, cache_dir);
+
+	while ((get_space_used_in_kb (cache_dir) > MAX_CACHE_SIZE_IN_KB) && (i < 100) ) {
+		system (command);
+		i++;
+	}
+
+	g_free (cache_dir);
+	debug (DEBUG_INFO, "Deleted %d files from cache to fit %ld kB limit.\n", i, MAX_CACHE_SIZE_IN_KB);
+}
+
+
 char*
 obtain_local_file (const char *remote_filename)
 {
 	char *md5_file;
 	char *cache_file;
 	char *local_file;
+	char *command;
 
 	/* If the file is local, simply return a copy of the filename, without
 	   any "file:///" prefix. */
-
+ 
 	if (is_local_file (remote_filename))
 		return get_local_path_from_uri (remote_filename);
 
 	/* If the file is remote, copy it to a local cache. */
-
 	md5_file = gnome_thumbnail_md5 (remote_filename);
+	if (md5_file == NULL)
+		return NULL;
+
 	cache_file = get_cache_full_path (md5_file, get_extension (remote_filename));
 	g_free (md5_file);
-
 	if (cache_file == NULL)
 		return NULL;
 
 	/* I can't imagine how the cache would be non-local, but check anyways */
 	g_assert (is_local_file (cache_file));
 
-	if (! path_exists (cache_file) || (get_file_mtime (cache_file) != get_file_mtime (remote_filename))) {
-		/* Move a new file into the cache.
-		 * The cache is pruned at startup. */
-
+	if (! path_exists (cache_file) || (get_file_mtime (cache_file) < get_file_mtime (remote_filename))) {
 		GnomeVFSURI    *source_uri = gnome_vfs_uri_new (remote_filename);
 		GnomeVFSURI    *target_uri = gnome_vfs_uri_new (cache_file);
 		GnomeVFSResult  result;
+	
+		/* delete some files if space is running out */
+		check_cache_space ();
 
+		/* Move a new file into the cache. */
 		result = gnome_vfs_xfer_uri (source_uri, target_uri,
 					     GNOME_VFS_XFER_DEFAULT | GNOME_VFS_XFER_FOLLOW_LINKS,
-						    GNOME_VFS_XFER_ERROR_MODE_ABORT,
-						    GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
-							 NULL,
-						  NULL);
+					     GNOME_VFS_XFER_ERROR_MODE_ABORT,
+					     GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
+					     NULL,
+					     NULL);
 
 		gnome_vfs_uri_unref (source_uri);
 		gnome_vfs_uri_unref (target_uri);
@@ -2510,11 +2573,18 @@ obtain_local_file (const char *remote_filename)
 			g_free (cache_file);
        			return NULL;
 		}
+
+		debug (DEBUG_INFO, "Copied %s into cache.\n", remote_filename);
 	}
 
 	local_file = get_local_path_from_uri (cache_file);
 	g_free (cache_file);
 
+	/* update mtimes so cache pruning works properly (delete oldest first) */
+	command = g_strdup_printf ("touch %s", local_file);
+	system (command);
+	g_free (command);
+
 	return local_file;
 }
 
@@ -2561,11 +2631,14 @@ get_pixbuf_using_external_converter (const char *url,
 	char	   *cache_file_full;
 	char	   *cache_file_esc;
 	char	   *input_file_esc;
-	char	   *command;
+	char	   *command = NULL;
 	GdkPixbuf  *pixbuf = NULL;
 	gboolean    is_raw;
 	gboolean    is_hdr;
 	gboolean    is_tiff;
+	gboolean    is_thumbnail;
+
+	is_thumbnail = requested_width_if_used > 0;
 
 	path = gnome_vfs_unescape_string (url, NULL);
 
@@ -2577,13 +2650,20 @@ get_pixbuf_using_external_converter (const char *url,
 
 	input_file_esc = shell_escape (path);
 
-	if (is_raw || is_tiff)
-		/* Same file used for RAW thumbnailing and full image loading */
+	/* The output filename, and its persistence, depend on the input file
+	   type, and whether or not a thumbnail has been requested. */
+
+	if ((is_tiff || is_raw) && !is_thumbnail)
+		/* Full-sized converted TIFF or RAW files */
 		cache_file_full = get_cache_full_path (md5_file, "conv.pnm");
-	else if (is_hdr && requested_width_if_used > 0)
-		/* HDR: separate files for thumbnails, full images */
+	else if ((is_tiff || is_raw) && is_thumbnail)
+		/* RAW: thumbnails generated in pnm format. The converted file is later removed. */
+		cache_file_full = get_cache_full_path (md5_file, "conv-thumb.pnm");
+	else if (is_hdr && is_thumbnail)
+		/* HDR: thumbnails generated in tiff format. The converted file is later removed. */
 		cache_file_full = get_cache_full_path (md5_file, "conv-thumb.tiff");
 	else
+		/* Full-sized converted HDR files */
 		cache_file_full = get_cache_full_path (md5_file, "conv.tiff");
 
 	cache_file = g_strdup (remove_host_from_uri (cache_file_full));
@@ -2604,14 +2684,50 @@ get_pixbuf_using_external_converter (const char *url,
 	    (get_file_mtime (path) > get_file_mtime (cache_file))) {
 
 		if (is_raw) {
-			/* RAW files - dcraw doesn't support thumbnailing very
-			   elegantly (even with the -e option), so we just convert
-			   the full file (which will speed up image loading). */
-			command = g_strconcat ( "dcraw -c ",
-						input_file_esc,
-						" > ",
-						cache_file_esc,
-						NULL );
+			if (is_thumbnail) {
+				char *first_part;
+			       	char *jpg_thumbnail;
+				char *tiff_thumbnail;
+				char *ppm_thumbnail;
+				char *thumb_command;
+
+				/* Check for an embedded thumbnail first */
+				thumb_command = g_strdup_printf ("dcraw -e %s", input_file_esc);
+                		if (gnome_vfs_is_executable_command_string (thumb_command))
+			        	system (thumb_command);
+				g_free (thumb_command);
+
+				first_part = remove_extension_from_path (path);
+				jpg_thumbnail = g_strdup_printf ("%s.thumb.jpg",first_part);
+				tiff_thumbnail = g_strdup_printf ("%s.thumb.tiff",first_part);
+				ppm_thumbnail = g_strdup_printf ("%s.thumb.ppm",first_part);
+
+				if (path_exists (jpg_thumbnail)) {
+					g_free (cache_file);
+					cache_file = g_strdup (jpg_thumbnail);
+				} else if (path_exists (tiff_thumbnail)) {
+                                        g_free (cache_file);
+                                        cache_file = g_strdup (tiff_thumbnail);
+				} else if (path_exists (ppm_thumbnail)) {
+                                        g_free (cache_file);
+                                        cache_file = g_strdup (ppm_thumbnail);
+				} else {
+					/* No embedded thumbnail. Read the whole file. */
+					/* Add -h option to speed up thumbnail generation. */
+					command = g_strdup_printf ("dcraw -w -c -h %s > %s",
+								   input_file_esc,
+								   cache_file_esc);
+				}
+				g_free (first_part);
+				g_free (jpg_thumbnail);
+				g_free (tiff_thumbnail);
+				g_free (ppm_thumbnail);
+			} else {
+				/* -w option = camera-specified white balance */
+                                command = g_strdup_printf ("dcraw -w -c %s > %s",
+                                                           input_file_esc,
+                                                           cache_file_esc);
+			}
 		}
 
 		if (is_hdr) {
@@ -2620,7 +2736,7 @@ get_pixbuf_using_external_converter (const char *url,
 			   thumbnailing as a special case. */
 			char *resize_command;
 
-			if (requested_width_if_used > 0)
+			if (is_thumbnail)
 				resize_command = g_strdup_printf (" | pfssize --maxx %d --maxy %d",
 								  requested_width_if_used,
 								  requested_height_if_used);
@@ -2648,20 +2764,22 @@ get_pixbuf_using_external_converter (const char *url,
 						    cache_file_esc);
 		}
 
-		if (gnome_vfs_is_executable_command_string (command))
-		       	system (command);
-
-		g_free (command);
+		if (command != NULL) {
+			if (gnome_vfs_is_executable_command_string (command))
+			       	system (command);
+			g_free (command);
+		}
 	}
 
 	if (path_is_file (cache_file))
 		pixbuf = gdk_pixbuf_new_from_file (cache_file, NULL);
 
-	/* Thumbnail files are already cached, so delete the conversion cache copies,
-	   except for raw files (because the same cache file is used for raw
- 	   thumbnails and raw full images). */
-	if (requested_width_if_used > 0 && !is_raw)
+	/* Thumbnail files are already cached, so delete the conversion cache copies */
+	if (is_thumbnail)
 		file_unlink (cache_file);
+	else
+		/* delete some files if space is running out */
+		check_cache_space ();
 
 	g_free (cache_file);
 	g_free (cache_file_esc);
@@ -2675,7 +2793,8 @@ get_pixbuf_using_external_converter (const char *url,
 static GdkPixbuf*
 gth_pixbuf_new_from_video (const char             *path,
 			   GnomeThumbnailFactory  *factory,
-			   GError                **error)
+			   GError                **error,
+			   const char	          *mime_type)
 {
       	GdkPixbuf *pixbuf = NULL;
 	time_t     mtime;
@@ -2704,8 +2823,8 @@ gth_pixbuf_new_from_video (const char             *path,
 	else {
 		pixbuf = gnome_thumbnail_factory_generate_thumbnail (factory,
 								     real_path,
-								     get_mime_type (real_path));
-		if (pixbuf != NULL)
+								     mime_type);
+		if (pixbuf != NULL) 
 			gnome_thumbnail_factory_save_thumbnail (factory,
 								pixbuf,
 								real_path,
@@ -2744,7 +2863,8 @@ gth_pixbuf_new_from_uri (const char  *uri,
 	/* Raw thumbnails - using libopenraw is much faster than using dcraw for
 	   thumbnails. Use libopenraw for full raw images too, once it matures. */
 	if ((pixbuf == NULL) &&
-	    mime_type_is_raw (mime_type) && (requested_width_if_used > 0))
+	    mime_type_is_raw (mime_type) && 
+	    (requested_width_if_used > 0))
 		pixbuf = or_gdkpixbuf_extract_thumbnail (local_file, requested_width_if_used);
 #endif
 
@@ -2759,7 +2879,27 @@ gth_pixbuf_new_from_uri (const char  *uri,
 							      requested_height_if_used);
 
 	/* Otherwise, use standard gdk_pixbuf loaders */
-	if (pixbuf == NULL)
+	if (pixbuf == NULL && (requested_width_if_used > 0)) {
+		int w, h;
+		
+		if (gdk_pixbuf_get_file_info (local_file, &w, &h) == NULL) {
+			w = -1;
+			h = -1;
+		}
+		
+		/* scale the image only if the original size is larger than
+		 * the requested size. */
+		if ((w > requested_width_if_used) || (h > requested_height_if_used))
+			pixbuf = gdk_pixbuf_new_from_file_at_scale (local_file,
+                        	                                    requested_width_if_used,
+                                	                            requested_height_if_used,
+                                        	                    TRUE,
+                                                	            error);
+		else
+			pixbuf = gdk_pixbuf_new_from_file (local_file, error);
+	}
+	else if (pixbuf == NULL)
+		/* otherwise, no scaling required */
 		pixbuf = gdk_pixbuf_new_from_file (local_file, error);
 
 	g_free (local_file);
@@ -2785,7 +2925,7 @@ gth_pixbuf_animation_new_from_uri (const char 	          *filename,
 	/* The video thumbnailer can handle VFS URIs directly */
 
 	if (mime_type_is_video (mime_type) && (factory != NULL)) {
-		pixbuf = gth_pixbuf_new_from_video (filename, factory, error);
+		pixbuf = gth_pixbuf_new_from_video (filename, factory, error, mime_type);
 		if (pixbuf == NULL)
 			return NULL;
 		animation = gdk_pixbuf_non_anim_new (pixbuf);
@@ -2800,24 +2940,6 @@ gth_pixbuf_animation_new_from_uri (const char 	          *filename,
 	if (local_file == NULL)
 		return NULL;
 
-	/* The jpeg thumbnailer can handle VFS URIs directly, but it is
-	   actually 3 times slower than copying it to a local cache.
-	   (Tested with ~ 3.7 MB jpeg files over ssh:// on DSL lines). */
-
-	/* Thumbnailing mode is signaled by requested_width_if_used > 0. */
-	if (mime_type_is (mime_type, "image/jpeg") && (requested_width_if_used > 0)) {
-		pixbuf = f_load_scaled_jpeg (local_file,
-					     requested_width_if_used,
-					     requested_height_if_used,
-					     NULL, NULL);
-		if (pixbuf == NULL)
-			return NULL;
-		animation = gdk_pixbuf_non_anim_new (pixbuf);
-		g_object_unref (pixbuf);
-		g_free (local_file);
-		return animation;
-	}
-
 	/* gifs: use gdk_pixbuf_animation_new_from_file */
 	if (mime_type_is (mime_type, "image/gif")) {
 		animation = gdk_pixbuf_animation_new_from_file (local_file, error);
@@ -2828,7 +2950,7 @@ gth_pixbuf_animation_new_from_uri (const char 	          *filename,
 	/* All other file types, or if previous methods fail: read in a
 	   non-animated pixbuf, and convert to a single-frame animation. */
 	if (pixbuf == NULL) {
-		char *local_uri = escape_uri(local_file);
+		char *local_uri = escape_uri (local_file);
 		pixbuf = gth_pixbuf_new_from_uri (local_uri,
 						  error,
 						  requested_width_if_used,
@@ -2895,3 +3017,118 @@ read_dot_hidden_file (const char *uri)
 	return hidden_files;
 }
 
+
+char *
+xdg_user_dir_lookup (const char *type)
+{
+	/* This function is used by gthumb to determine the default "PICTURES"
+	   directory for imports. The actually directory name is localized. 
+	   Bug 425365. */
+
+	FILE *file;
+	char *home_dir, *config_home, *config_file;
+	char buffer[512];
+	char *user_dir;
+	char *p, *d;
+	int len;
+	int relative;
+
+	home_dir = getenv ("HOME");
+	if (home_dir == NULL)
+		return strdup ("/tmp");
+
+	config_home = getenv ("XDG_CONFIG_HOME");
+	if (config_home == NULL || config_home[0] == 0) {
+		config_file = malloc (strlen (home_dir) + strlen ("/.config/user-dirs.dirs") + 1);
+		strcpy (config_file, home_dir);
+		strcat (config_file, "/.config/user-dirs.dirs");
+	} else {
+		config_file = malloc (strlen (config_home) + strlen ("/user-dirs.dirs") + 1);
+		strcpy (config_file, config_home);
+		strcat (config_file, "/user-dirs.dirs");
+	}
+
+	file = fopen (config_file, "r");
+	free (config_file);
+	if (file == NULL)
+		goto error;
+
+	user_dir = NULL;
+
+	while (fgets (buffer, sizeof (buffer), file)) {
+		/* Remove newline at end */
+		len = strlen (buffer);
+		if (len > 0 && buffer[len-1] == '\n')
+		buffer[len-1] = 0;
+      
+		p = buffer;
+		while (*p == ' ' || *p == '\t')
+			p++;
+      
+		if (strncmp (p, "XDG_", 4) != 0)
+			continue;
+		p += 4;
+
+		if (strncmp (p, type, strlen (type)) != 0)
+			continue;
+		p += strlen (type);
+
+		if (strncmp (p, "_DIR", 4) != 0)
+		continue;
+		p += 4;
+
+		while (*p == ' ' || *p == '\t')
+			p++;
+
+		if (*p != '=')
+			continue;
+		p++;
+      
+		while (*p == ' ' || *p == '\t')
+			p++;
+
+		if (*p != '"')
+			continue;
+		p++;
+      
+		relative = 0;
+		if (strncmp (p, "$HOME/", 6) == 0) {
+			p += 6;
+			relative = 1;
+		} else if (*p != '/')
+			continue;
+      
+		if (relative) {
+			user_dir = malloc (strlen (home_dir) + 1 + strlen (p) + 1);
+			strcpy (user_dir, home_dir);
+			strcat (user_dir, "/");
+		} else {
+			user_dir = malloc (strlen (p) + 1);
+			*user_dir = 0;
+		}
+      
+		d = user_dir + strlen (user_dir);
+		while (*p && *p != '"')	{
+			if ((*p == '\\') && (*(p+1) != 0))
+				p++;
+			*d++ = *p++;
+		}
+		*d = 0;
+	}  
+	
+	fclose (file);
+
+	if (user_dir)
+		return user_dir;
+
+error:
+	 /* Special case desktop for historical compatibility */
+	if (strcmp (type, "DESKTOP") == 0) {
+		user_dir = malloc (strlen (home_dir) + strlen ("/Desktop") + 1);
+		strcpy (user_dir, home_dir);
+		strcat (user_dir, "/Desktop");
+		return user_dir;
+	} else
+		return strdup (home_dir);
+}
+
