@@ -318,11 +318,13 @@ add_monitor_event (GthMonitor                *monitor,
 	if (event_type == GNOME_VFS_MONITOR_EVENT_CREATED) {
 		if (path_is_file (path))
 			type = MONITOR_EVENT_FILE_CREATED;
-		else
+		else if (path_is_dir (path))
 			type = MONITOR_EVENT_DIR_CREATED;
+		else
+			return;
 
 	} else if (event_type == GNOME_VFS_MONITOR_EVENT_DELETED) {
-		if (file_is_image_or_video (path, TRUE))
+		if (file_is_image_video_or_audio (path, TRUE))
 			type = MONITOR_EVENT_FILE_DELETED;
 		else
 			type = MONITOR_EVENT_DIR_DELETED;
